/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: IEjbJarClassMetadata.java 4840 2009-03-26 12:59:59Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.api;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.util.ee.metadata.common.api.interfaces.IEJBInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvEntryAccess;
import org.ow2.util.ee.metadata.common.api.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.ITransactionAttribute;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IConcurrencyManagementType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IDependsOn;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJSingleton;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateless;
import org.ow2.util.ee.metadata.ws.api.IWebservicesCommonClassMetadata;

/**
 * IEjbJarClassMetadata.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Gael Lalire
 */
public interface IEjbJarClassMetadata<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends
        ITransactionAttribute,
        ILock,
        IEJBInterceptors,
        IAnnotationSecurityRolesAllowed,
        IAnnotationSecurityPermitAll,
        IInterceptorExcludeDefaultInterceptors,
        IWebservicesCommonClassMetadata<C, M, F>,
        IEnvEntryAccess,
        Serializable {

    /**
     * @return name of the bean (associated to this metadata).
     */
    String getClassName();

    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    void setLocalInterfaces(final IJLocal jLocal);

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    void setRemoteInterfaces(final IJRemote jRemote);

    /**
     * @return the local interfaces of this class.
     */
    IJLocal getLocalInterfaces();

    /**
     * @return the remote interfaces of this class.
     */
    IJRemote getRemoteInterfaces();

    /**
     * @return true if the class is a stateless class
     */
    boolean isStateless();

    /**
     * @return true if the class is a singleton class
     */
    boolean isSingleton();

    /**
     * @return true if the class is a stateful class
     */
    boolean isStateful();

    /**
     * @return true if the class is a session bean class
     */
    boolean isSession();

    /**
     * @return true if the class is an MDB class
     */
    boolean isMdb();

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    void setClassType(final ClassType cType);

    /**
     * @return Message driven attribute.
     */
    IJMessageDriven getJMessageDriven();

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    void setJMessageDriven(final IJMessageDriven messageDriven);

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    String getRemoteHome();

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    void setRemoteHome(final String remoteHome);

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    String getLocalHome();

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    void setLocalHome(final String localHome);

    /**
     * @return transaction management type from @see TransactionManagementType.
     */
    TransactionManagementType getTransactionManagementType();

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value.
     *      (BEAN, CONTAINER)
     */
    void setTransactionManagementType(final TransactionManagementType transactionManagementType);

    /**
     * @return transaction management type from
     * @see javax.ejb.ConcurrencyManagementType.
     */
    IConcurrencyManagementType getConcurrencyManagementType();

    /**
     * Sets concurrency management type.
     * @see javax.ejb.ConcurrencyManagementType
     * @param concurrencyManagementType value.
     *      BEAN,CONCURRENCY_NOT_ALLOWED, CONTAINER
     */
    void setConcurrencyManagementType(final IConcurrencyManagementType concurrencyManagementType);

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    TransactionAttributeType getTransactionAttributeType();


    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType);

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    IJInterceptors getAnnotationInterceptors();

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors);

    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    ApplicationException getApplicationException();

    /**
     * Sets the object representing the &#64;{@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    void setApplicationException(final ApplicationException applicationException);

    /**
     * @return true if the class is a Bean
     */
    boolean isBean();

    /**
     * @return array of interfaces name.
     */
    String[] getInterfaces();

    /**
     * @return the super class name.
     */
    String getSuperName();

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getExternalUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor classes).
     * @param externalUserInterceptors list of interceptors that enhancer will use.
     */
    void setExternalUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors);

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (bean classes)
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getInternalUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will use.
     */
    void setInternalUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors);

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors();

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors);

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    IJCommonBean getJCommonBean();

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    void setJCommonBean(final IJCommonBean commonBean);

    /**
     * @return the attributes for a Stateful
     */
    IJStateful getJStateful();

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    void setJStateful(final IJStateful jStateful);

    /**
     * @return the attributes for a Stateless
     */
    IJStateless getJStateless();

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    void setJStateless(final IJStateless jStateless);


    /**
     * @return the attributes for a Singleton
     */
    IJSingleton getJSingleton();

    /**
     * Sets the attributes for a Singleton.
     * @param jSingleton the attributes
     */
    void setJSingleton(final IJSingleton jSingleton);

    /**
     * @return true if the class has been modified.
     */
    boolean wasModified();

    /**
     * Defines that this class has been modified.
     */
    void setModified();

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    void setDeclareRoles(final List<String> declareRoles);

    /**
     * @return the list of roles declared on this class.
     */
    List<String> getDeclareRoles();


    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     * @return list of &lt;security-role-ref&gt; elements.
     */
    List<ISecurityRoleRef> getSecurityRoleRefList();

    /**
     * Sets the list of &lt;security-role-ref&gt; element.
     * @param securityRoleRefList the list of &lt;security-role-ref&gt; element
     */
    void setSecurityRoleRefList(final List<ISecurityRoleRef> securityRoleRefList);

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    void setRolesAllowed(final List<String> rolesAllowed);

    /**
     * @return the list of roles allowed on this class/method.
     */
    List<String> getRolesAllowed();

    /**
     * This class has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    void setPermitAll(final boolean permitAll);

    /**
     * @return true if PermitAll annotation.
     */
    boolean hasPermitAll();

    /**
     * @return true if this class won't use default interceptors.
     */
    boolean isExcludedDefaultInterceptors();

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors);

    /**
     * @return true if the interceptors have been ordered for this class.
     */
    boolean isOrderedInterceptors();

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    void setOrderedInterceptors(final boolean orderedInterceptors);

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    void setRemoteInterface(final String remoteInterface);

    /**
     * @return the name of the remote interface.
     */
    String getRemoteInterface();

    /**
     * construct a new JClass.
     * @param array interface list
     */
    void setInterfaces(final String[] array);

    /**
     * @return the deployable
     */
    D getEjbJarDeployableMetadata();

    /**
     * Set the deployable metadata.
     * @param ejbJarArchiveMetadata the given deployable metadata
     */
    void setEjbJarArchiveMetadata(D ejbJarArchiveMetadata);

    /**
     * @return the name of the bean on which we are linked.
     * This is used for cloned classmetadata that are for the classes.
     */
    String getLinkedBean();

    /**
     * This class is linked to the given bean.
     * @param linkedBeanName the bean name
     */
    void setLinkedBean(final String linkedBeanName);

    /**
     * Get a classmetadata linked from this one.
     * @param className the name of the class that is wanted
     * @return the class metadata
     */
    C getLinkedClassMetadata(final String className);

    /**
     * This class needs eager initialization.
     * @param startup the boolean value.
     */
    void setStartup(final boolean startup);

    /**
     * @return true if this class(singleton) needs eager initialization.
     */
    boolean isStartup();

    /**
     * @return object representing list of &#64;
     *         {@link javax.interceptor.DependsOn}.
     */
    IDependsOn getDependsOn();

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.DependsOn} annotation.
     * @param dependsOn list of beans
     */
    void setDependsOn(final IDependsOn dependsOn);

}
